@@ -1,5 +1,6 @@
 import uuid
 from math import ceil
+from typing import List, Union
 
 from gpt_image.entry import Entry
 from gpt_image.geometry import Geometry
@@ -18,9 +19,9 @@ class Partition:
 
     def __init__(
         self,
-        name: str = None,
+        name: str = "",
         size: int = 0,
-        partition_guid: uuid.UUID = None,
+        partition_guid: Union[None, uuid.UUID] = None,
         alignment: int = 8,
     ):
         """Initialize Partition Object
@@ -33,7 +34,7 @@ class Partition:
         entry array.
 
         Attributes:
-          size: partition size in Bytes
+            size: partition size in Bytes
         """
         # create an empty partition object
         self.type_guid = Entry(0, 16, 0)
@@ -42,22 +43,23 @@ class Partition:
         self.last_lba = Entry(40, 8, 0)
         self.attribute_flags = Entry(48, 8, 0)
         self.partition_name = Entry(56, 72, 0)
-
         # if name is set, this isn't an empty partition. Set relevant fields
+        # @TODO: don't base an empty partition off of the name attribute
         if name:
             self.type_guid.data = uuid.UUID(Partition.LINUX_FILE_SYSTEM).bytes_le
             if not partition_guid:
                 self.partition_guid.data = uuid.uuid4().bytes_le
             else:
                 self.partition_guid.data = partition_guid.bytes_le
-            b_name = bytes(name, encoding="utf_16_le")
-            # ensure the partition name is padded
-            self.partition_name.data = b_name + bytes(72 - len(b_name))
+            # the partition name is stored as utf_16_le
+            self.partition_name.data = bytes(name, encoding="utf_16_le")
 
         self.alignment = alignment
         self.size = size
 
-        self.partition_fields = [
+    @property
+    def byte_structure(self) -> bytes:
+        part_fields = [
             self.type_guid,
             self.partition_guid,
             self.first_lba,
@@ -65,19 +67,47 @@ class Partition:
             self.attribute_flags,
             self.partition_name,
         ]
-
-    def as_bytes(self) -> bytes:
-        """Return the partition as bytes"""
-        byte_list = [x.data for x in self.partition_fields]
+        byte_list = [x.data_bytes for x in part_fields]
         return b"".join(byte_list)
 
+    def read(self, partition_bytes):
+        """Unmarshal bytes to Partition Object"""
+        self.type_guid.data = partition_bytes[
+            self.type_guid.offset : self.type_guid.offset + self.type_guid.length
+        ]
+        self.partition_guid.data = partition_bytes[
+            self.partition_guid.offset : self.partition_guid.offset
+            + self.partition_guid.length
+        ]
+        self.first_lba.data = partition_bytes[
+            self.first_lba.offset : self.first_lba.offset + self.first_lba.length
+        ]
+        self.last_lba.data = partition_bytes[
+            self.last_lba.offset : self.last_lba.offset + self.last_lba.length
+        ]
+        self.attribute_flags.data = partition_bytes[
+            self.attribute_flags.offset : self.attribute_flags.offset
+            + self.attribute_flags.length
+        ]
+        part_name_b = partition_bytes[
+            self.partition_name.offset : self.partition_name.offset
+            + self.partition_name.length
+        ]
+        # partition name is stored as UTF-16-LE padded to 72 bytes
+        self.partition_name.data = part_name_b.decode(encoding="utf_16_le").rstrip(
+            "\x00"
+        )
+
 
 class PartitionEntryArray:
     """Stores the Partition objects for a Table"""
 
-    def __init__(self, geometry: Geometry, entry_size: int = 128):
-        self.entries = [Partition()] * entry_size
-        self._geometry = geometry
+    EntryCount = 128
+    EntryLength = 128
+
+    def __init__(self, geometry: Geometry):
+        self.entries: List[Partition] = []
+        self._geometry: Geometry = geometry
 
     def add(self, partition: Partition) -> None:
         """Add a partition to the entries
@@ -85,12 +115,9 @@ class PartitionEntryArray:
         Appends the Partition to the next available entry. Calculates the
         LBA's
         """
-        partition.first_lba.data = (self._get_first_lba(partition)).to_bytes(
-            8, "little"
-        )
-        partition.last_lba.data = (self._get_last_lba(partition)).to_bytes(8, "little")
-        entry_index = self._get_next_partition()
-        self.entries[entry_index] = partition
+        partition.first_lba.data = self._get_first_lba(partition)
+        partition.last_lba.data = self._get_last_lba(partition)
+        self.entries.append(partition)
 
     def _get_first_lba(self, partition: Partition) -> int:
         """Calculate the first LBA of a new partition
@@ -109,11 +136,11 @@ class PartitionEntryArray:
 
         largest_lba = 0
         for part in self.entries:
-            lba = int.from_bytes(part.last_lba.data, byteorder="little")
-            if lba > largest_lba:
+            lba = part.last_lba.data
+            if int(lba) > int(largest_lba):
                 largest_lba = lba
         last_lba = 33 if largest_lba == 0 else largest_lba
-        return next_lba(last_lba, partition.alignment)
+        return next_lba(int(last_lba), partition.alignment)
 
     def _get_last_lba(self, partition: Partition) -> int:
         """Calculate the last LBA of a new partition
@@ -126,22 +153,18 @@ class PartitionEntryArray:
         ), "Partition smaller than sector size"
 
         # round the LBA up to ensure our LBA will hold the partition
-        lba = ceil(partition.size / self._geometry.sector_size)
-        f_lba = int.from_bytes(partition.first_lba.data, byteorder="little")
+        lba = int(ceil(partition.size / self._geometry.sector_size))
+        f_lba = int(partition.first_lba.data)
         return (f_lba + lba) - 1
 
-    def _get_next_partition(self) -> int:
-        """Return the index of the next unused partition"""
-        for idx, part in enumerate(self.entries):
-            # return the first partition index that has no name
-            if int.from_bytes(part.partition_name.data, byteorder="little") == 0:
-                return idx
-        raise AttributeError("No partition entries available")
-
-    def as_bytes(self) -> bytes:
-        """Represent as bytes
-
-        Return the entire partition entry list as bytes
-        """
-        parts = [x.as_bytes() for x in self.entries]
-        return b"".join(parts)
+    @property
+    def byte_structure(self) -> bytes:
+        """Convert the Partition Array to its byte structure"""
+        parts = [x.byte_structure for x in self.entries]
+        part_bytes = b"".join(parts)
+        # pad the rest with zeros
+        padded = part_bytes + b"\x00" * (
+            (PartitionEntryArray.EntryCount * PartitionEntryArray.EntryLength)
+            - len(part_bytes)
+        )
+        return padded
